import org.graphstream.algorithm.Dijkstra;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.Graph;
import org.graphstream.graph.implementations.Graphs;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.GraphParseException;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;


public class TestDijkstra
{
    public static void main(String[] args) throws IOException, GraphParseException {

        int nbSommet   = 1000;
        int degreMoyen = 6;
        //Création des graphes
        Graph g = GenererGraphe(nbSommet,degreMoyen);
        //Graph g = GenererGraphe("grapheCours");
        Graph g2 = Graphs.clone(g);

        Calendar c=Calendar.getInstance();
        c.setTime(new Date());
        //Mon Dijkstra

        long timeProgramme  = System.currentTimeMillis();
        //Lancement du Disjkstra naïve programmé
        MonDijkstra.Dijkstra(g);
        timeProgramme = System.currentTimeMillis() - timeProgramme;
        System.out.println("Disjktra programmé : ");
        System.out.println(MonDijkstra.affichage(g));

        //GraphStream Dijkstra

        long timeStreamDijstra = System.currentTimeMillis();

        //Lancement du Disjkstra
        Dijkstra dijkstra = new Dijkstra(Dijkstra.Element.EDGE, null, "distance");
        dijkstra.init(g2);
        dijkstra.setSource(g2.getNode(0));
        dijkstra.compute();

        timeStreamDijstra = System.currentTimeMillis() - timeStreamDijstra;

        // Affiche le graphe
        System.out.println("Disjktra de graphStream : ");
        System.out.println(MonDijkstra.affichage(g2, dijkstra));

        //Affiche le temps d'execution
        System.out.println("Temps d'exécution de Dijstra (programmé)   : " + timeProgramme     + " ms");

        System.out.println("Temps d'exécution de Dijstra (graphstream) : " + timeStreamDijstra + " ms");


    }

    public static Graph GenererGraphe(int nbSommet, double degreMoyen)
    {
        Graph g = new SingleGraph("Random");
        g.setAttribute("ui.quality");
        g.setAttribute("ui.antialias");

        Generator gen = new RandomGenerator(degreMoyen,true, true);
        gen.addSink(g);
        gen.begin();
        for(int i=0; i<nbSommet; i++)
            gen.nextEvents();

        gen.end();

        g.getNode(0).setAttribute("poids",0);

        for (int i = 1; i < g.getNodeCount(); i++)
            g.getNode(i).setAttribute("poids",Integer.MAX_VALUE);

        for (int i = 0; i < g.getEdgeCount(); i++)
            g.getEdge(i).setAttribute("distance",(int)(Math.random()*99)+1);
        return g;
    }

    public static Graph GenererGraphe(String nomFichier) throws IOException, GraphParseException {
        Graph g = new SingleGraph("grapheAvecFichier");
        g.read("data/"+nomFichier+".dgs");
        System.setProperty("org.graphstream.ui", "swing");

        g.setAttribute("ui.quality");
        g.setAttribute(("ui.antialias"));

        g.getNode(0).setAttribute("poids",0);

        for (int i = 1; i < g.getNodeCount(); i++)
            g.getNode(i).setAttribute("poids",Integer.MAX_VALUE);

        System.setProperty("org.graphstream.ui", "swing");
        return g;
    }

}
